@@ -5,7 +5,7 @@
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
     Copyright 2013, Nikita Zlobin
-    Copyright 2014-2016, Will Godfrey & others
+    Copyright 2014-2017, Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -21,7 +21,9 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    This file is derivative of ZynAddSubFX original code, last modified September 2016
+    This file is derivative of ZynAddSubFX original code.
+
+    Modified August 2017
 */
 
 #include <iostream>
@@ -57,7 +59,7 @@ static char prog_doc[] =
     "Copyright 2002-2009 Nasca Octavian Paul and others, "
     "Copyright 2009-2011 Alan Calvert, "
     "Copyright 20012-2013 Jeremy Jongepier and others, "
-    "Copyright 20014-2016 Will Godfrey and others";
+    "Copyright 20014-2017 Will Godfrey and others";
 string argline = "Yoshimi " + (string) YOSHIMI_VERSION + "\nBuild Number " + to_string(BUILD_NUMBER);
 const char* argp_program_version = argline.c_str();
 
@@ -100,7 +102,8 @@ Config::Config(SynthEngine *_synth, int argc, char **argv) :
     stateChanged(false),
     restoreJackSession(false),
     runSynth(true),
-    VirKeybLayout(1),
+    finishedCLI(true),
+    VirKeybLayout(0),
     audioEngine(DEFAULT_AUDIO),
     midiEngine(DEFAULT_MIDI),
     audioDevice("default"),
@@ -120,6 +123,7 @@ Config::Config(SynthEngine *_synth, int argc, char **argv) :
     hideErrors(0),
     showTimes(0),
     logXMLheaders(0),
+    xmlmax(0),
     configChanged(false),
     rtprio(40),
     midi_bank_root(0), // 128 is used as 'disabled'
@@ -128,10 +132,11 @@ Config::Config(SynthEngine *_synth, int argc, char **argv) :
     enable_part_on_voice_load(1),
     ignoreResetCCs(false),
     monitorCCin(false),
+    showLearnedCC(true),
     single_row_panel(1),
     NumAvailableParts(NUM_MIDI_CHANNELS),
     currentPart(0),
-    padApply(0xffff),
+    lastPatchSet(-1),
     channelSwitchType(0),
     channelSwitchCC(128),
     channelSwitchValue(0),
@@ -220,7 +225,7 @@ bool Config::Setup(int argc, char **argv)
         midiDevice = "";
     loadCmdArgs(argc, argv);
     Oscilsize = nearestPowerOf2(Oscilsize, MAX_AD_HARMONICS * 2, 16384);
-    Buffersize = nearestPowerOf2(Buffersize, 16, 1024);
+    Buffersize = nearestPowerOf2(Buffersize, 16, 4096);
     //Log(asString(Oscilsize));
     //Log(asString(Buffersize));
     if (loadDefaultState && !restoreState)
@@ -287,18 +292,10 @@ string Config::testCCvalue(int cc)
             result = "mod wheel";
             break;
 
-        case 10:
-            result = "panning";
-            break;
-
         case 11:
             result = "expression";
             break;
 
-        case 38:
-            result = "data lsb";
-            break;
-
         case 71:
             result = "filter Q";
             break;
@@ -316,7 +313,7 @@ string Config::testCCvalue(int cc)
             break;
 
         case 77:
-            result = "resonance centre";
+            result = "resonance center";
             break;
 
         case 78:
@@ -343,6 +340,10 @@ string Config::masterCCtest(int cc)
             result = "volume";
             break;
 
+        case 10:
+            result = "panning";
+            break;
+
         case 38:
             result = "data lsb";
             break;
@@ -523,17 +524,17 @@ bool Config::extractBaseParameters(XMLwrapper *xml)
         return false;
     }
     Samplerate = xml->getpar("sample_rate", Samplerate, 44100, 192000);
-    Buffersize = xml->getpar("sound_buffer_size", Buffersize, 16, 1024);
+    Buffersize = xml->getpar("sound_buffer_size", Buffersize, 16, 4096);
     Oscilsize = xml->getpar("oscil_size", Oscilsize, MAX_AD_HARMONICS * 2, 16384);
     GzipCompression = xml->getpar("gzip_compression", GzipCompression, 0, 9);
     showGui = xml->getparbool("enable_gui", showGui);
     showSplash = xml->getparbool("enable_splash", showSplash);
     showCLI = xml->getparbool("enable_CLI", showCLI);
-    if (!showGui && !showCLI)
+    /*if (!showGui && !showCLI)
     {
         showGui = true;
         showCLI = true; // sanity check!
-    }
+    }*/
     xml->exitbranch(); // BaseParameters
     return true;
 }
@@ -556,7 +557,8 @@ bool Config::extractConfigData(XMLwrapper *xml)
     hideErrors = xml->getpar("hide_system_errors", hideErrors, 0, 1);
     showTimes = xml->getpar("report_load_times", showTimes, 0, 1);
     logXMLheaders = xml->getpar("report_XMLheaders", logXMLheaders, 0, 1);
-    VirKeybLayout = xml->getpar("virtual_keyboard_layout", VirKeybLayout, 0, 10);
+    VirKeybLayout = xml->getpar("virtual_keyboard_layout", VirKeybLayout, 1, 6) - 1;
+    xmlmax = xml->getpar("full_parameters", xmlmax, 0, 1);
 
     // get preset dirs
     int count = 0;
@@ -603,6 +605,8 @@ bool Config::extractConfigData(XMLwrapper *xml)
     EnableProgChange = 1 - xml->getpar("ignore_program_change", EnableProgChange, 0, 1); // inverted for Zyn compatibility
     enable_part_on_voice_load = xml->getpar("enable_part_on_voice_load", enable_part_on_voice_load, 0, 1);
     ignoreResetCCs = xml->getpar("ignore_reset_all_CCs",ignoreResetCCs,0, 1);
+    monitorCCin = xml->getparbool("monitor-incoming_CCs", monitorCCin);
+    showLearnedCC = xml->getparbool("open_editor_on_learned_CC", showLearnedCC);
 
     //misc
     checksynthengines = xml->getpar("check_pad_synth", checksynthengines, 0, 1);
@@ -614,24 +618,29 @@ bool Config::extractConfigData(XMLwrapper *xml)
 }
 
 
-void Config::saveConfig(void)
+bool Config::saveConfig(void)
 {
+    bool result = false;
     xmlType = XML_CONFIG;
     XMLwrapper *xmltree = new XMLwrapper(synth);
     if (!xmltree)
     {
-        Log("saveConfig failed xmltree allocation");
-        return;
+        Log("saveConfig failed xmltree allocation", 2);
+        return result;
     }
     addConfigXML(xmltree);
     string resConfigFile = ConfigFile;
 
     if (xmltree->saveXMLfile(resConfigFile))
+    {
         configChanged = false;
+        result = true;
+    }
     else
-        Log("Failed to save config to " + resConfigFile);
+        Log("Failed to save config to " + resConfigFile, 2);
 
     delete xmltree;
+    return result;
 }
 
 
@@ -643,7 +652,8 @@ void Config::addConfigXML(XMLwrapper *xmltree)
     xmltree->addpar("hide_system_errors", hideErrors);
     xmltree->addpar("report_load_times", showTimes);
     xmltree->addpar("report_XMLheaders", logXMLheaders);
-    xmltree->addpar("virtual_keyboard_layout", VirKeybLayout);
+    xmltree->addpar("virtual_keyboard_layout", VirKeybLayout + 1);
+    xmltree->addpar("full_parameters", xmlmax);
 
     for (int i = 0; i < MAX_PRESET_DIRS; ++i)
     {
@@ -673,6 +683,8 @@ void Config::addConfigXML(XMLwrapper *xmltree)
     xmltree->addpar("ignore_program_change", (1 - EnableProgChange));
     xmltree->addpar("enable_part_on_voice_load", enable_part_on_voice_load);
     xmltree->addpar("ignore_reset_all_CCs",ignoreResetCCs);
+    xmltree->addparbool("monitor-incoming_CCs", monitorCCin);
+    xmltree->addparbool("open_editor_on_learned_CC",showLearnedCC);
     xmltree->addpar("check_pad_synth", checksynthengines);
     xmltree->addpar(string("root_current_ID"), synth->ReadBankRoot());
     xmltree->addpar(string("bank_current_ID"), synth->ReadBank());
@@ -680,24 +692,30 @@ void Config::addConfigXML(XMLwrapper *xmltree)
 }
 
 
-void Config::saveSessionData(string savefile)
+bool Config::saveSessionData(string savefile)
 {
-    string ext = ".state";
-    if (savefile.rfind(ext) != (savefile.length() - 6))
-        savefile += ext;
+    savefile = setExtension(savefile, "state");
     synth->getRuntime().xmlType = XML_STATE;
     XMLwrapper *xmltree = new XMLwrapper(synth);
     if (!xmltree)
     {
-        Log("saveSessionData failed xmltree allocation", 1);
-        return;
+        Log("saveSessionData failed xmltree allocation", 3);
+        return false;
     }
+    bool ok = true;
     addConfigXML(xmltree);
     synth->add2XML(xmltree);
+    synth->midilearn.insertMidiListData(false, xmltree);
     if (xmltree->saveXMLfile(savefile))
-        Log("Session data saved to " + savefile);
+        Log("Session data saved to " + savefile, 2);
     else
-        Log("Failed to save session data to " + savefile, 1);
+    {
+        ok = false;
+        Log("Failed to save session data to " + savefile, 2);
+    }
+    if (xmltree)
+        delete xmltree;
+    return ok;
 }
 
 
@@ -705,22 +723,22 @@ bool Config::restoreSessionData(string sessionfile, bool startup)
 {
     XMLwrapper *xml = NULL;
     bool ok = false;
+
     if (sessionfile.size() && !isRegFile(sessionfile))
-        sessionfile += ".state";
+        sessionfile = setExtension(sessionfile, "state");
     if (!sessionfile.size() || !isRegFile(sessionfile))
     {
-        Log("Session file " + sessionfile + " not available", 1);
+        Log("Session file " + sessionfile + " not available", 2);
         goto end_game;
     }
     if (!(xml = new XMLwrapper(synth)))
     {
-        Log("Failed to init xmltree for restoreState", 1);
+        Log("Failed to init xmltree for restoreState", 3);
         goto end_game;
     }
-
     if (!xml->loadXMLfile(sessionfile))
     {
-        Log("Failed to load xml file " + sessionfile);
+        Log("Failed to load xml file " + sessionfile, 2);
         goto end_game;
     }
     if (startup)
@@ -729,10 +747,20 @@ bool Config::restoreSessionData(string sessionfile, bool startup)
     {
         ok = extractConfigData(xml); // this still needs improving
         if (ok)
-        {
+        { // mark as soon as anything changes
+            synth->getRuntime().stateChanged = true;
+            for (int npart = 0; npart < NUM_MIDI_PARTS; ++ npart)
+            {
+                synth->part[npart]->defaults();
+                synth->part[npart]->Prcvchn = npart % NUM_MIDI_CHANNELS;
+            }
             ok = synth->getfromXML(xml);
             if (ok)
-                synth->getRuntime().stateChanged = true;
+                synth->setAllPartMaps();
+            bool oklearn = synth->midilearn.extractMidiListData(false, xml);
+            if (oklearn)
+                synth->midilearn.updateGui(2);
+                // handles possibly undefined window
         }
     }
 
@@ -749,8 +777,11 @@ void Config::Log(string msg, char tostderr)
         return;
     if (showGui && !(tostderr & 1) && toConsole)
         LogList.push_back(msg);
+    else if (!tostderr & 1)
+        cout << msg << endl; // normal log
+
     else
-        cerr << msg << endl;
+        cerr << msg << endl; // error log
 }
 
 
@@ -1224,7 +1255,7 @@ void GuiThreadMsg::processGuiMessages()
             {
 
                 case GuiThreadMsg::UpdateMaster:
-                    guiMaster->refresh_master_ui();
+                    guiMaster->refresh_master_ui(msg->index);
                     break;
 
                 case GuiThreadMsg::UpdateConfig:
@@ -1266,6 +1297,11 @@ void GuiThreadMsg::processGuiMessages()
                         guiMaster->updateeffects(msg->index);
                     break;
 
+                case GuiThreadMsg::UpdateControllers:
+                    if (msg->data)
+                        guiMaster->updatecontrollers(msg->index);
+                    break;
+
                 case GuiThreadMsg::UpdateBankRootDirs:
                     if (msg->data)
                         guiMaster->updateBankRootDirs();
@@ -1293,6 +1329,7 @@ void GuiThreadMsg::processGuiMessages()
                 case GuiThreadMsg::GuiAlert:
                     if (msg->data)
                         guiMaster->ShowAlert(msg->index);
+                    break;
 
                 default:
                     break;
