@@ -4,7 +4,7 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2014-2015, Will Godfrey & others
+    Copyright 2014-2017, Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -20,7 +20,9 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    This file is derivative of ZynAddSubFX original code, last modified January 2015
+    This file is derivative of ZynAddSubFX original code.
+
+    Modified August 2017
 */
 
 #ifndef CONFIG_H
@@ -34,7 +36,6 @@
 using namespace std;
 
 #include "MusicIO/MusicClient.h"
-#include "Misc/HistoryListItem.h"
 #include "Misc/MiscFuncs.h"
 #include "FL/Fl.H"
 
@@ -61,10 +62,10 @@ class Config : public MiscFuncs
 
         string testCCvalue(int cc);
         string masterCCtest(int cc);
-        void saveConfig(void);
+        bool saveConfig(void);
         bool loadConfig(void);
-        void saveState() { saveSessionData(StateFile); }
-        void saveState(const string statefile)  { saveSessionData(statefile); }
+        //bool saveState() { return saveSessionData(StateFile); }
+        bool saveState(const string statefile)  { return saveSessionData(statefile); }
         bool loadState(const string statefile)
             { return restoreSessionData(statefile, false); }
         bool stateRestore(void)
@@ -103,7 +104,7 @@ class Config : public MiscFuncs
         static bool          showCLI;
 
         bool          runSynth;
-
+        bool          finishedCLI;
         int           VirKeybLayout;
 
         audio_drivers audioEngine;
@@ -131,6 +132,7 @@ class Config : public MiscFuncs
         bool          hideErrors;
         bool          showTimes;
         bool          logXMLheaders;
+        bool          xmlmax;
         bool          configChanged;
         int           rtprio;
         int           tempRoot;
@@ -141,10 +143,11 @@ class Config : public MiscFuncs
         int           enable_part_on_voice_load;
         bool          ignoreResetCCs;
         bool          monitorCCin;
+        bool          showLearnedCC;
         int           single_row_panel;
         int           NumAvailableParts;
         int           currentPart;
-        unsigned int  padApply;
+        int           lastPatchSet;
         unsigned char channelSwitchType;
         unsigned char channelSwitchCC;
         unsigned char channelSwitchValue;
@@ -154,25 +157,35 @@ class Config : public MiscFuncs
         unsigned char dataH;
         bool          nrpnActive;
 
-        struct IOdata{
-            unsigned char vectorXaxis[NUM_MIDI_CHANNELS];
-            unsigned char vectorYaxis[NUM_MIDI_CHANNELS];
-            unsigned char vectorXfeatures[NUM_MIDI_CHANNELS];
-            unsigned char vectorYfeatures[NUM_MIDI_CHANNELS];
-            unsigned char vectorXcc2[NUM_MIDI_CHANNELS];
-            unsigned char vectorYcc2[NUM_MIDI_CHANNELS];
-            unsigned char vectorXcc4[NUM_MIDI_CHANNELS];
-            unsigned char vectorYcc4[NUM_MIDI_CHANNELS];
-            unsigned char vectorXcc8[NUM_MIDI_CHANNELS];
-            unsigned char vectorYcc8[NUM_MIDI_CHANNELS];
+        struct{
+            unsigned char Xaxis[NUM_MIDI_CHANNELS];
+            unsigned char Yaxis[NUM_MIDI_CHANNELS];
+            unsigned char Xfeatures[NUM_MIDI_CHANNELS];
+            unsigned char Yfeatures[NUM_MIDI_CHANNELS];
+            unsigned char Xcc2[NUM_MIDI_CHANNELS];
+            unsigned char Ycc2[NUM_MIDI_CHANNELS];
+            unsigned char Xcc4[NUM_MIDI_CHANNELS];
+            unsigned char Ycc4[NUM_MIDI_CHANNELS];
+            unsigned char Xcc8[NUM_MIDI_CHANNELS];
+            unsigned char Ycc8[NUM_MIDI_CHANNELS];
+            string Name[NUM_MIDI_CHANNELS];
             int Part;
             int Controller;
-            bool vectorEnabled[NUM_MIDI_CHANNELS];
-        };
-
-        IOdata nrpndata;
+            bool Enabled[NUM_MIDI_CHANNELS];
+        }vectordata;
 
         list<string> LogList;
+
+        /*
+         * These replace local memory allocations that
+         * were being made every time an add or sub note
+         * was processed. Now global so treat with care!
+         */
+        float *genTmp1;
+        float *genTmp2;
+        float *genTmp3;
+        float *genTmp4;
+
         BodyDisposal *deadObjects;
 
     private:
@@ -181,7 +194,7 @@ class Config : public MiscFuncs
         bool extractBaseParameters(XMLwrapper *xml);
         bool extractConfigData(XMLwrapper *xml);
         void addConfigXML(XMLwrapper *xml);
-        void saveSessionData(string savefile);
+        bool saveSessionData(string savefile);
         bool restoreSessionData(string sessionfile, bool startup);
         int SSEcapability(void);
         void AntiDenormals(bool set_daz_ftz);
@@ -223,6 +236,7 @@ public:
         UpdatePartProgram,
         UpdateEffects,
         UpdateBankRootDirs,
+        UpdateControllers,
         RescanForBanks,
         RefreshCurBank,
         GuiAlert,
