@@ -2,7 +2,7 @@
     MiscFuncs.cpp
 
     Copyright 2010, Alan Calvert
-    Copyright 2014-2016, Will Godfrey
+    Copyright 2014-2017, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
@@ -15,7 +15,9 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with yoshimi.  If not, see <http://www.gnu.org/licenses/>.
+    along with yoshimi.  If not, see <http://www.gnu.org/licenses/>
+
+    Modifed July 2017
 */
 
 #include <sys/stat.h>
@@ -100,6 +102,8 @@ string MiscFuncs::asString(float n)
 string MiscFuncs::asLongString(float n)
 {
    ostringstream oss;
+   oss.precision(9);
+   oss.width(9);
    oss << n;
    return oss.str();
 }
@@ -109,7 +113,10 @@ string MiscFuncs::asHexString(int x)
 {
    ostringstream oss;
    oss << hex << x;
-   return string(oss.str());
+   string res = string(oss.str());
+   if (res.length() & 1)
+       return "0"+res;
+   return res;
 }
 
 
@@ -117,7 +124,10 @@ string MiscFuncs::asHexString(unsigned int x)
 {
    ostringstream oss;
    oss << hex << x;
-   return string(oss.str());
+   string res = string(oss.str());
+   if (res.length() & 1)
+       return "0"+res;
+   return res;
 }
 
 
@@ -130,6 +140,15 @@ float MiscFuncs::string2float(string str)
 }
 
 
+double MiscFuncs::string2double(string str)
+{
+    istringstream machine(str);
+    double dval;
+    machine >> dval;
+    return dval;
+}
+
+
 int MiscFuncs::string2int(string str)
 {
     istringstream machine(str);
@@ -258,7 +277,7 @@ string MiscFuncs::setExtension(string fname, string ext)
 
     string tmp;                         // return value
     size_t ext_pos = fname.rfind('.');  // period, if any
-    size_t slash_pos = fname.find('/'); // UNIX path-separator
+    size_t slash_pos = fname.rfind('/'); // UNIX path-separator
     if (slash_pos == string::npos)
     {
         // There are no slashes in the string, therefore the last period, if
@@ -330,7 +349,7 @@ char *MiscFuncs::skipChars(char *buf)
     {
         ++ buf;
     }
-    if (buf[0] == 0x20)
+    if (buf[0] == 0x20) // now find the next word (if any)
         buf = skipSpace(buf);
     return buf;
 }
@@ -365,7 +384,7 @@ bool MiscFuncs::matchnMove(int num , char *&pnt, const char *word)
  * the same 'live' ID, but if they do, the second one will get an
  * empty string.
  *
- * Both will block, but should be very quick;
+ * Both calls will block, but should be very quick;
  *
  * Normally a message will clear before the next one arrives so the
  * message numbers should remain very low even over multiple instances.
@@ -379,8 +398,9 @@ void MiscFuncs::miscMsgInit()
 
 int MiscFuncs::miscMsgPush(string _text)
 {
-    mutex mtx;
-    mtx.lock();
+    if (_text.empty())
+        return 255;
+    sem_wait(&miscmsglock);
 
     string text = _text;
     list<string>::iterator it = miscList.begin();
@@ -391,6 +411,7 @@ int MiscFuncs::miscMsgPush(string _text)
         if ( *it == "")
         {
             *it = text;
+            //cout << "Msg In " << int(idx) << " >" << text << "<" << endl;
             break;
         }
         ++ it;
@@ -398,20 +419,21 @@ int MiscFuncs::miscMsgPush(string _text)
     }
     if (it == miscList.end())
     {
-        cout << "List full :(" << endl;
+        cerr << "miscMsg list full :(" << endl;
         idx = -1;
     }
 
     int result = idx; // in case of a new entry before return
-    mtx.unlock();
+    sem_post(&miscmsglock);
     return result;
 }
 
 
 string MiscFuncs::miscMsgPop(int _pos)
 {
-    mutex mtx;
-    mtx.lock();
+    if (_pos >= 255)
+        return "";
+    sem_wait(&miscmsglock);
 
     int pos = _pos;
     list<string>::iterator it = miscList.begin();
@@ -420,14 +442,19 @@ string MiscFuncs::miscMsgPop(int _pos)
     while(it != miscList.end())
     {
         if (idx == pos)
+        {
+            //cout << "Msg Out " << int(idx) << " >" << *it << "<" << endl;
             break;
+        }
         ++ it;
         ++ idx;
     }
     string result = "";
     if (idx == pos)
-        swap( result, *it); // in case of a new entry before return
-    mtx.unlock();
+    {
+        swap (result, *it); // in case of a new entry before return
+    }
+    sem_post(&miscmsglock);
     return result;
 }
 
