@@ -2,6 +2,7 @@
     MiscFuncs.h
 
     Copyright 2010, Alan Calvert
+    Copyright 2014-2017, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
@@ -15,6 +16,8 @@
 
     You should have received a copy of the GNU General Public License
     along with yoshimi.  If not, see <http://www.gnu.org/licenses/>.
+
+    Modifed February 2017
 */
 
 #ifndef MISCFUNCS_H
@@ -23,6 +26,7 @@
 #include <cmath>
 #include <string>
 #include <list>
+#include <semaphore.h>
 
 using namespace std;
 
@@ -31,8 +35,8 @@ static list<string> miscList;
 class MiscFuncs
 {
     public:
-        MiscFuncs() { }
-        ~MiscFuncs() { }
+        MiscFuncs() {sem_init(&miscmsglock, 0, 1);}
+        ~MiscFuncs() {sem_destroy(&miscmsglock);}
         string asString(int n);
         string asString(long long n);
         string asString(unsigned long n);
@@ -45,6 +49,7 @@ class MiscFuncs
         string asHexString(unsigned int x);
 
         static float string2float(string str);
+        static double string2double(string str);
         static int string2int(string str);
         static int string2int127(string str);
         static unsigned int string2uint(string str);
@@ -78,6 +83,7 @@ class MiscFuncs
 
         float dB2rap(float dB);
         float rap2dB(float rap);
+        sem_t miscmsglock;
 };
 
 void invSignal(float *sig, size_t len);
