@@ -5,7 +5,7 @@
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009, James Morris
     Copyright 2009-2011, Alan Calvert
-    Copyright 2014-2016, Will Godfrey
+    Copyright 2014-2017, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -21,7 +21,9 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    This file is derivative of ZynAddSubFX original code, modified December 2016
+    This file is derivative of ZynAddSubFX original code.
+
+    Modified August 2017
 */
 
 #include <cstring>
@@ -50,7 +52,6 @@ using namespace std;
 Part::Part(Microtonal *microtonal_, FFTwrapper *fft_, SynthEngine *_synth) :
     microtonal(microtonal_),
     fft(fft_),
-    partMuted(0),
     killallnotes(false),
     synth(_synth)
 {
@@ -108,7 +109,6 @@ Part::Part(Microtonal *microtonal_, FFTwrapper *fft_, SynthEngine *_synth) :
     cleanup();
     Pname.clear();
 
-    oldvolumel = oldvolumer = 0.5f;
     lastnote = -1;
     lastpos = 0; // lastpos will store previously used NoteOn(...)'s pos.
     lastlegatomodevalid = false; // To store previous legatomodevalid value.
@@ -124,9 +124,12 @@ void Part::defaults(void)
     Ppolymode = 1;
     Plegatomode = 0;
     setVolume(96);
+    TransVolume = 128; // ensure it always gets set
     Pkeyshift = 64;
+    PmapOffset = 0;
     Prcvchn = 0;
     setPan(Ppanning = 64);
+    TransPanning = 128; // ensure it always gets set
     Pvelsns = 64;
     Pveloffs = 64;
     Pkeylimit = 20;
@@ -141,9 +144,9 @@ void Part::setNoteMap(int keyshift)
 {
     for (int i = 0; i < 128; ++i)
         if (Pdrummode)
-            PnoteMap[i] = microtonal->getFixedNoteFreq(i);
+            PnoteMap[128 - PmapOffset + i] = microtonal->getFixedNoteFreq(i);
         else
-            PnoteMap[i] = microtonal->getNoteFreq(i, keyshift + synth->Pkeyshift - 64);
+            PnoteMap[128 - PmapOffset + i] = microtonal->getNoteFreq(i, keyshift + synth->Pkeyshift - 64);
 }
 
 
@@ -249,7 +252,6 @@ Part::~Part()
 // Note On Messages
 void Part::NoteOn(int note, int velocity, int masterkeyshift)
 {
-//    if (!Pnoteon || note < Pminkey || note > Pmaxkey)
     if (note < Pminkey || note > Pmaxkey)
         return;
 
@@ -366,7 +368,7 @@ void Part::NoteOn(int note, int velocity, int masterkeyshift)
 
         // initialise note frequency
         float notebasefreq;
-        if ((notebasefreq = PnoteMap[note]) < 0.0f)
+        if ((notebasefreq = PnoteMap[PmapOffset + note]) < 0.0f)
             return; // the key is not mapped
 
         // Humanise
@@ -611,7 +613,7 @@ void Part::NoteOn(int note, int velocity, int masterkeyshift)
                     }
                     if (range)
                     {
-                        vel = truevel * ((float)position / range);
+                        vel = truevel * (float(position) / float(range));
                         //cout << item << "  " << vel << endl;
                     }
                 }
@@ -850,10 +852,7 @@ void Part::MonoMemRenote(void)
 {
     unsigned char mmrtempnote = monomemnotes.back(); // Last list element.
     monomemnotes.pop_back(); // We remove it, will be added again in NoteOn(...).
-/*    if (Pnoteon == 0)
-        RelaseNotePos(lastpos);
-    else*/
-        NoteOn(mmrtempnote, monomem[mmrtempnote].velocity,
+    NoteOn(mmrtempnote, monomem[mmrtempnote].velocity,
                monomem[mmrtempnote].mkeyshift);
 }
 
@@ -953,13 +952,6 @@ void Part::setkeylimit(unsigned char Pkeylimit_)
 // Compute Part samples and store them in the partoutl[] and partoutr[]
 void Part::ComputePartSmps(void)
 {
-    if (isMuted())
-    {
-        memset(partoutl, 0, synth->p_bufferbytes);
-        memset(partoutr, 0, synth->p_bufferbytes);
-        return;
-    }
-
     int k;
     int noteplay; // 0 if there is nothing activated
     for (int nefx = 0; nefx < NUM_PART_EFX + 1; ++nefx)
@@ -1055,6 +1047,15 @@ void Part::ComputePartSmps(void)
             KillNotePos(k);
     }
 
+    for (int item = 0; item < NUM_KIT_ITEMS; ++item)
+    {
+        if (kit[item].adpars)
+            kit[item].adpars->postrender();
+        if (kit[item].subpars)
+            kit[item].subpars->postrender();
+        if (kit[item].padpars)
+            kit[item].padpars->postrender();
+    }
     // Apply part's effects and mix them
     for (int nefx = 0; nefx < NUM_PART_EFX; ++nefx)
     {
@@ -1105,8 +1106,14 @@ void Part::ComputePartSmps(void)
 // Parameter control
 void Part::setVolume(float value)
 {
-    Pvolume = (int)value;
-    volume  = dB2rap((value - 96.0f) / 96.0f * 40.0f) * ctl->expression.relvolume;
+    Pvolume = value;
+}
+
+
+void Part::checkVolume(float step)
+{
+    TransVolume += step;
+    volume = dB2rap((TransVolume - 96.0f) / 96.0f * 40.0f);
 }
 
 
@@ -1116,10 +1123,17 @@ void Part::setDestination(int value)
 }
 
 
-void Part::setPan(char value)
+void Part::setPan(float value)
 {
     Ppanning = value;
-    float t = ((Ppanning > 0) ? (float)(Ppanning - 1) : 0.0f) / 126.0f;
+}
+
+
+void Part::checkPanning(float step)
+{
+    float t;
+    TransPanning += step;
+    t = ((TransPanning > 0) ? (TransPanning - 1) : 0.0f) / 126.0f;
     pangainL = cosf(t * HALFPI);
     pangainR = cosf((1.0f - t) * HALFPI);
 }
@@ -1253,9 +1267,7 @@ void Part::add2XMLinstrument(XMLwrapper *xml)
 void Part::add2XML(XMLwrapper *xml)
 {
     // parameters
-    xml->addparbool("enabled", Penabled);
-    //if (!Penabled && xml->minimal)
-        //return;
+    xml->addparbool("enabled", (Penabled == 1));
 
     xml->addpar("volume", Pvolume);
     xml->addpar("panning", Ppanning);
@@ -1268,7 +1280,6 @@ void Part::add2XML(XMLwrapper *xml)
     xml->addpar("velocity_sensing", Pvelsns);
     xml->addpar("velocity_offset", Pveloffs);
 
-//    xml->addparbool("note_on", Pnoteon);
     xml->addparbool("poly_mode", Ppolymode);
     xml->addpar("legato_mode", Plegatomode);
     xml->addpar("key_limit", Pkeylimit);
@@ -1435,7 +1446,7 @@ void Part::getfromXMLinstrument(XMLwrapper *xml)
 
 void Part::getfromXML(XMLwrapper *xml)
 {
-    Penabled = xml->getparbool("enabled", Penabled);
+    Penabled = (xml->getparbool("enabled", Penabled) == 1);
 
     setVolume(xml->getpar127("volume", Pvolume));
     setPan(xml->getpar127("panning", Ppanning));
@@ -1443,13 +1454,12 @@ void Part::getfromXML(XMLwrapper *xml)
     Pminkey = xml->getpar127("min_key", Pminkey);
     Pmaxkey = xml->getpar127("max_key", Pmaxkey);
     Pkeyshift = xml->getpar("key_shift", Pkeyshift, MIN_KEY_SHIFT + 64, MAX_KEY_SHIFT + 64);
-    setNoteMap(Pkeyshift - 64);
+
     Prcvchn = xml->getpar127("rcv_chn", Prcvchn);
 
     Pvelsns = xml->getpar127("velocity_sensing", Pvelsns);
     Pveloffs = xml->getpar127("velocity_offset", Pveloffs);
 
-//    Pnoteon = xml->getparbool("note_on", Pnoteon);
     Ppolymode = xml->getparbool("poly_mode", Ppolymode);
     Plegatomode = xml->getparbool("legato_mode", Plegatomode); // older versions
     if (!Plegatomode)
@@ -1481,3 +1491,159 @@ void Part::getfromXML(XMLwrapper *xml)
         xml->exitbranch();
     }
 }
+
+
+void Part::getLimits(CommandBlock *getData)
+{
+    unsigned int type = getData->data.type;
+    int control = getData->data.control;
+    int npart = getData->data.part;
+
+    // defaults
+    int min = 0;
+    int def = 640;
+    int max = 127;
+    //cout << "part control " << to_string(control) << endl;
+    if ((control >= 128 && control <= 168) || control == 224)
+    {
+        ctl->getLimits(getData);
+        return;
+    }
+
+    switch (control)
+    {
+        case 0:
+            type &= 0x3f;
+            type |= 0x40;
+            def = 960;
+            break;
+
+        case 1:
+        case 4:
+            type |= 0x40;
+            break;
+
+        case 2:
+            type &= 0x3f;
+            type |= 0x40;
+            break;
+
+        case 5:
+            min = 1;
+            def = 10;
+            max = 16;
+            break;
+
+        case 6:
+            def = 0;
+            max = 2;
+            break;
+
+        case 7:
+            type |= 0x40;
+        case 57:
+            def = 0;
+            max = 1;
+            break;
+
+        case 8:
+            if (npart == 0)
+                def = 10;
+            else
+                def = 0;
+            max = 1;
+            break;
+
+        case 9:
+            def = 0;
+            max = 1;
+            break;
+
+        case 16:
+            def = 0;
+            break;
+
+        case 17:
+            def = 1270;
+            break;
+
+        case 18:
+        case 19:
+        case 20:
+        case 96:
+            min = 0;
+            def = 0;
+            max = 0;
+            break;
+
+        case 33:
+            def = 200;
+            max = 60;
+            break;
+
+        case 35:
+            min = -36;
+            def = 0;
+            max = 36;
+            break;
+
+        case 40:
+        case 41:
+        case 42:
+        case 43:
+            type |= 0x40;
+            def = 0;
+            break;
+
+        case 48:
+            def = 0;
+            max = 50;
+            break;
+
+        case 58:
+            def = 0;
+            max = 3;
+            break;
+        case 120:
+            min = 1;
+            def = 10;
+            max = 3;
+            break;
+
+        // the following are learnable MIDI controllers
+        case 192:
+        case 197:
+        case 198:
+            type |= 0x40;
+            break;
+
+        case 194:
+            type |= 0x40;
+            def = 1270;
+            break;
+
+        // these haven't been done
+        case 193:
+            break;
+        case 195:
+            break;
+        case 196:
+            break;
+
+        case 255: // number of parts!
+            min = 16;
+            def = 160;
+            max = 64;
+            break;
+
+        default:
+            min = -1;
+            def = -10;
+            max = -1;
+            break;
+    }
+    getData->data.type = type;
+    getData->limits.min = min;
+    getData->limits.def = def;
+    getData->limits.max = max;
+}
