@@ -4,6 +4,7 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
+    Copyright 2014-2017, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -19,7 +20,9 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    This file is derivative of ZynAddSubFX original code, modified April 2011
+    This file is derivative of ZynAddSubFX original code.
+
+    Modified June 2017
 */
 
 #ifndef XML_WRAPPER_H
@@ -54,6 +57,8 @@ class XMLwrapper : private MiscFuncs
         void addpar(const string& name, int val); // add simple parameter: name, value
         void addparreal(const string& name, float val);
 
+        void addpardouble(const string& name, double val);
+
         void addparbool(const string& name, int val); // 1 => "yes", else "no"
 
 
@@ -109,7 +114,7 @@ class XMLwrapper : private MiscFuncs
         float getparreal(const string& name, float defaultpar,
                          float min, float max);
 
-        bool minimal; // false if all parameters will be stored (used only for clipboard)
+        bool minimal; // false if all parameters will be stored
 
         struct {
             unsigned char ADDsynth_used;
@@ -143,6 +148,10 @@ class XMLwrapper : private MiscFuncs
         mxml_node_t *addparams2(const string& name, const string& par1, const string& val1,
                                 const string& par2, const string& val2);
 
+        mxml_node_t *addparams3(const string& name, const string& par1, const string& val1,
+                                const string& par2, const string& val2,
+                                const string& par3, const string& val3);
+
         // this is used to store the parents
         mxml_node_t *parentstack[STACKSIZE];
         int stackpos;
