@@ -2,7 +2,7 @@
     JackEngine.cpp
 
     Copyright 2009-2011, Alan Calvert
-    Copyright 2014, Will Godfrey
+    Copyright 2014-2017, Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
@@ -17,7 +17,7 @@
     You should have received a copy of the GNU General Public License
     along with yoshimi.  If not, see <http://www.gnu.org/licenses/>.
 
-    Modified October 2014
+    Modified April 2017
 */
 
 #include <errno.h>
@@ -67,7 +67,7 @@ bool JackEngine::connectServer(string server)
     else
     {
         synth->getRuntime().Log("Failed to open jack client on server " + server);
-        splashMessages.push_back("Can't connect to jack :(");
+//        splashMessages.push_back("Can't connect to jack :(");
     }
     return false;
 }
@@ -136,8 +136,6 @@ bool JackEngine::Start(void)
 {
     bool jackPortsRegistered = true;
     internalbuff = synth->getRuntime().Buffersize;
-    //Andrew Deryabin: use default error callback function provided by jack
-    //jack_set_error_function(_errorCallback);
     jack_set_xrun_callback(jackClient, _xrunCallback, this);
     #if defined(JACK_SESSION)
         if (jack_set_session_callback
@@ -227,22 +225,20 @@ void JackEngine::registerAudioPort(int partnum)
     {
         if(audio.ports [portnum] != NULL)
         {
-            //port already registered
-            //synth->getRuntime().Log("Jack port " + asString(partnum) + " already registered!");
+            synth->getRuntime().Log("Jack port " + asString(partnum) + " already registered!", 2);
             return;
         }
         /* This has a hack to stop all enabled parts from resistering
          * individual ports (at startup) if part is not configured for
          * direct O/P.
-         * Also, at startup there seems to be no way to enable main L/R
-         * without also enabling part 0 direct :(
          */
-        if(synth->part [partnum] && synth->partonoffRead(partnum) && (synth->part [partnum]->Paudiodest > 1 || partnum == 0))
+        string portName;
+        if(synth->part [partnum] && synth->partonoffRead(partnum) && (synth->part [partnum]->Paudiodest > 1))
         {
-            string portName = "track_" + asString(partnum + 1) + "_r";
-            audio.ports[portnum + 1] = jack_port_register(jackClient, portName.c_str(), JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
             portName = "track_" + asString(partnum + 1) + "_l";
             audio.ports[portnum] = jack_port_register(jackClient, portName.c_str(), JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
+            portName = "track_" + asString(partnum + 1) + "_r";
+            audio.ports[portnum + 1] = jack_port_register(jackClient, portName.c_str(), JACK_DEFAULT_AUDIO_TYPE, JackPortIsOutput, 0);
 
             if(audio.ports [portnum])
             {
@@ -321,10 +317,19 @@ bool JackEngine::connectJackPorts(void)
         const char *port_name = jack_port_name(audio.ports[port + NUM_MIDI_PARTS * 2]);
         if ((ret = jack_connect(jackClient, port_name, playback_ports[port])))
         {
+            if(ret == EEXIST)
+            {
+            synth->getRuntime().Log(string(port_name)
+                        + " is already connected to jack port " + string(playback_ports[port])
+                        + ", status " + asString(ret));
+            }
+            else
+            {
             synth->getRuntime().Log("Cannot connect " + string(port_name)
                         + " to jack port " + string(playback_ports[port])
                         + ", status " + asString(ret));
             return false;
+            }
         }
     }
     return true;
@@ -362,8 +367,10 @@ int JackEngine::processCallback(jack_nframes_t nframes)
     bool okmidi = true;
 
     if (midi.port)
+        // input exists, using jack midi
         okmidi = processMidi(nframes);
-    if (audio.ports[0] && audio.ports[1])
+    if (audio.ports[NUM_MIDI_PARTS * 2] && audio.ports[NUM_MIDI_PARTS * 2 + 1])
+        // (at least) main outputs exist, using jack audio
         okaudio = processAudio(nframes);
     return (okaudio && okmidi) ? 0 : -1;
 }
@@ -399,17 +406,19 @@ bool JackEngine::processAudio(jack_nframes_t nframes)
         return false;
     }
 
-    int framesize = sizeof(float) * nframes;
+    int framesize;
     if (nframes < internalbuff)
     {
+        framesize = sizeof(float) * nframes;
         synth->MasterAudio(zynLeft, zynRight, nframes);
         sendAudio(framesize, 0);
     }
     else
     {
+        framesize = sizeof(float) * internalbuff;
         for (unsigned int pos = 0; pos < nframes; pos += internalbuff)
         {
-        synth->MasterAudio(zynLeft, zynRight, 0);
+        synth->MasterAudio(zynLeft, zynRight, internalbuff);
         sendAudio(framesize, pos);
         }
     }
@@ -462,104 +471,13 @@ bool JackEngine::processMidi(jack_nframes_t nframes)
     unsigned int idx;
     jack_midi_event_t jEvent;
     jack_nframes_t eventCount = jack_midi_get_event_count(portBuf);
-    unsigned char channel, note, velocity;
-    int ctrltype;
-    int par;
-    unsigned int ev;
+
     for(idx = 0; idx < eventCount; ++idx)
     {
         if(!jack_midi_event_get(&jEvent, portBuf, idx))
         {
-            par = 0;
-            if (jEvent.size < 1 || jEvent.size > 4)
-                continue; // no interest in zero sized or long events
-            channel = jEvent.buffer[0] & 0x0F;
-            switch ((ev = jEvent.buffer[0] & 0xF0))
-            {
-                case 0x01: // modulation wheel or lever
-                    ctrltype = C_modwheel;
-                    par = jEvent.buffer[2];
-                    setMidiController(channel, ctrltype, par);
-                    break;
-
-                case 0x07: // channel volume (formerly main volume)
-                    ctrltype = C_volume;
-                    par = jEvent.buffer[2];
-                    setMidiController(channel, ctrltype, par);
-                    break;
-
-                case 0x0B: // expression controller
-                    ctrltype = C_expression;
-                    par = jEvent.buffer[2];
-                    setMidiController(channel, ctrltype, par);
-                    break;
-
-                case 0x78: // all sound off
-                    ctrltype = C_allsoundsoff;
-                    setMidiController(channel, ctrltype, 0);
-                    break;
-
-                case 0x79: // reset all controllers
-                    ctrltype = C_resetallcontrollers;
-                    setMidiController(channel, ctrltype, 0);
-                    break;
-
-                case 0x7B:  // all notes off
-                    ctrltype = C_allnotesoff;
-                    setMidiController(channel, ctrltype, 0);
-                    break;
-
-                case 0x80: // note-off
-                    note = jEvent.buffer[1];
-                    setMidiNote(channel, note);
-                    break;
-
-                case 0x90: // note-on
-                    if ((note = jEvent.buffer[1])) // skip note == 0
-                    {
-                        velocity = jEvent.buffer[2];
-                        setMidiNote(channel, note, velocity);
-                    }
-                    break;
-
-                case 0xA0: // key aftertouch
-                    ctrltype = C_keypressure;
-                    // need to work out how to use key values >> j Event.buffer[1]
-                    par = jEvent.buffer[2];
-                    setMidiController(channel, ctrltype, par);
-                    break;
-
-                case 0xB0: // controller
-                    ctrltype = jEvent.buffer[1];//getMidiController(jEvent.buffer[1]);
-                    par = jEvent.buffer[2];
-                    setMidiController(channel, ctrltype, par);
-                    break;
-
-                case 0xC0: // program change
-                    ctrltype = C_programchange;
-                    par = jEvent.buffer[1];
-                    setMidiProgram(channel, par);
-                    break;
-
-                case 0xD0: // channel aftertouch
-                    ctrltype = C_channelpressure;
-                    par = jEvent.buffer[1];
-                    setMidiController(channel, ctrltype, par);
-                    break;
-
-                case 0xE0: // pitch bend
-                    ctrltype = C_pitchwheel;
-                    par = ((jEvent.buffer[2] << 7) | jEvent.buffer[1]) - 8192;
-                    setMidiController(channel, ctrltype, par);
-                    break;
-
-                case 0xF0: // system exclusive
-                    break;
-
-                default: // wot, more? commented out some progs spam us :(
-                    synth->getRuntime().Log("other event: " + asString((int)ev), 1);
-                    break;
-            }
+            if (jEvent.size >= 1 && jEvent.size <= 4) // no interest in zero sized or long events
+                setMidi(jEvent.buffer[0], jEvent.buffer[1], jEvent.buffer[2]);
         }
     }
     return true;
@@ -573,12 +491,6 @@ int JackEngine::_xrunCallback(void *arg)
 }
 
 
-void JackEngine::_errorCallback(const char *msg)
-{
-    //synth->getRuntime().Log("Jack reports error: " + string(msg));
-}
-
-
 bool JackEngine::latencyPrep(void)
 {
 #if defined(JACK_LATENCY)  // >= 0.120.1 API
@@ -592,13 +504,13 @@ bool JackEngine::latencyPrep(void)
 
 #else // < 0.120.1 API
 
-    if (jack_port_set_latency && audio.ports[0] && audio.ports[1])
+    for (int i = 0; i < 2 * NUM_MIDI_PARTS + 2; ++i)
     {
-        jack_port_set_latency(audio.ports[0], jack_get_buffer_size(jackClient));
-        jack_port_set_latency(audio.ports[1], jack_get_buffer_size(jackClient));
-        if (jack_recompute_total_latencies)
-            jack_recompute_total_latencies(jackClient);
+        if (jack_port_set_latency && audio.ports[i])
+            jack_port_set_latency(audio.ports[i], jack_get_buffer_size(jackClient));
     }
+    if (jack_recompute_total_latencies)
+        jack_recompute_total_latencies(jackClient);
     return true;
 
 #endif
@@ -641,15 +553,15 @@ void JackEngine::latencyCallback(jack_latency_callback_mode_t mode)
 {
     if (mode == JackCaptureLatency)
     {
-        if (audio.ports[0] && audio.ports[1])
+        for (int i = 0; i < 2 * NUM_MIDI_PARTS + 2; ++i)
         {
-            jack_latency_range_t range[2];
-            for (int i = 0; i < 2; ++i)
+            jack_latency_range_t range;
+            if (audio.ports[i])
             {
-                jack_port_get_latency_range(audio.ports[i], mode, &range[i]);
-                range[i].min++;
-                range[i].max += audio.jackNframes;
-                jack_port_set_latency_range(audio.ports[i], JackPlaybackLatency, &range[i]);
+                jack_port_get_latency_range(audio.ports[i], mode, &range);
+                range.min++;
+                range.max += audio.jackNframes;
+                jack_port_set_latency_range(audio.ports[i], JackPlaybackLatency, &range);
             }
         }
     }
