@@ -2,6 +2,7 @@
     MusicClient.h
 
     Copyright 2009-2011, Alan Calvert
+    Copyright 2016, Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can
     redistribute it and/or modify it under the terms of the GNU General
@@ -15,12 +16,15 @@
 
     You should have received a copy of the GNU General Public License
     along with yoshimi.  If not, see <http://www.gnu.org/licenses/>.
+
+    Modified December 2016
 */
 
 #include "MusicIO/MusicClient.h"
 #include "Misc/SynthEngine.h"
 #include "MusicIO/AlsaEngine.h"
 #include "MusicIO/JackEngine.h"
+#include <iostream>
 #include <stdlib.h>
 #include <set>
 
@@ -86,8 +90,6 @@ MusicClient::MusicClient(SynthEngine *_synth, audio_drivers _audioDrv, midi_driv
     :synth(_synth), timerThreadId(0), timerWorking(false),
     audioDrv(_audioDrv), midiDrv(_midiDrv), audioIO(0), midiIO(0)
 {
-    //synth->getRuntime().Samplerate = NMC_SRATE;
-
     for(int i = 0; i < NUM_MIDI_PARTS + 1; i++)
     {
         buffersL [i] = new float [synth->getRuntime().Buffersize];
