@@ -4,6 +4,7 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
+    Copyright 2017, Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -19,10 +20,13 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    This file is derivative of ZynAddSubFX original code, modified April 2011
+    This file is derivative of ZynAddSubFX original code
+
+    Modified April 2017
 */
 
 #include <cmath>
+#include <iostream>
 
 using namespace std;
 
@@ -395,3 +399,118 @@ void Controller::getfromXML(XMLwrapper *xml)
     resonancecenter.depth=xml->getpar127("resonance_center_depth",resonancecenter.depth);
     resonancebandwidth.depth=xml->getpar127("resonance_bandwidth_depth",resonancebandwidth.depth);
 }
+
+
+void Controller::getLimits(CommandBlock *getData)
+{
+    int control = getData->data.control;
+    //cout << "ctl control " << to_string(control) << endl;
+
+    // defaults
+    int type = 0x80;
+    int min = 0;
+    int def = 640;
+    int max = 127;
+
+    switch (control)
+    {
+        case 128:
+            min = 64;
+            def = 960;
+            break;
+        case 129:
+            def = 10;
+            max = 1;
+            break;
+        case 130:
+            type |= 0x40;
+            max = 64;
+            break;
+        case 131:
+            def = 800;
+            break;
+        case 132:
+            def = 0;
+            max = 1;
+            break;
+        case 133:
+            type |= 0x40;
+            break;
+        case 134:
+            def = 0;
+            max = 1;
+            break;
+        case 135:
+            def = 10;
+            max = 1;
+            break;
+        case 136:
+            def = 10;
+            max = 1;
+            break;
+        case 137:
+            def = 10;
+            max = 1;
+            break;
+        case 138:
+            type |= 0x40;
+            min = -6400;
+            def = 0;
+            max = 6400;
+            break;
+        case 139:
+            break;
+        case 140:
+            break;
+        case 144:
+            break;
+        case 145:
+            break;
+        case 160:
+            type |= 0x40;
+            min = 0;
+            break;
+        case 161:
+            type |= 0x40;
+            break;
+        case 162:
+            type |= 0x40;
+            def = 30;
+            break;
+        case 163:
+            min = 0;
+            max = 1;
+            def = 10;
+            break;
+        case 164:
+            def = 0;
+            max = 1;
+            break;
+        case 165:
+            type |= 0x40;
+            def = 800;
+            break;
+        case 166:
+            type |= 0x40;
+            def = 900;
+            break;
+        case 168:
+            def = 10;
+            max = 1;
+            break;
+        case 224:
+            def = 0;
+            max = 0;
+            break;
+
+        default:
+            min = -1;
+            def = -10;
+            max = -1;
+            break;
+    }
+    getData->data.type = type;
+    getData->limits.min = min;
+    getData->limits.def = def;
+    getData->limits.max = max;
+}
