@@ -19,7 +19,9 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    This file is a derivative of a ZynAddSubFX original, modified December 2016
+    This file is a derivative of a ZynAddSubFX original.
+
+    Modified August 2017
 */
 
 #include <cmath>
@@ -217,8 +219,11 @@ float PADnoteParameters::getprofile(float *smp, int size)
         float x_before_freq_mult = x;
         // do the frequency multiplier
         x *= freqmult;
+
         // do the modulation of the profile
-        x += sinf(x_before_freq_mult * 3.1415926f * modfreq) * modpar1;
+        //x += sinf(x_before_freq_mult * 3.1415926f * modfreq) * modpar1;
+        x += sinf(x_before_freq_mult * PI * modfreq) * modpar1; // should be the same
+
         x = fmodf(x + 1000.0f, 1.0f) * 2.0f - 1.0f;
         // this is the base function of the profile
         float f;
@@ -938,19 +943,241 @@ void PADnoteParameters::getfromXML(XMLwrapper *xml)
 void PADnoteParameters::getLimits(CommandBlock *getData)
 {
     int control = getData->data.control;
+
+    // defaults
+    int type = 0;
+    int min = 0;
+    int def = 640;
+    int max = 127;
+
     switch (control)
     {
+        case 0:
+            type |= 0x40;
+            def = 900;
+            break;
+
+        case 1:
+            type |= 0x40;
+            def = 720;
+            break;
+
+        case 2:
+            type |= 0x40;
+            break;
+
+        case 8:
+            type |= 0x40;
+            def = 0;
+            max = 1;
+            break;
+
         case 16:
-            getData->limits.max = 1000;
+            def = 5000;
+            max = 1000;
             break;
+
+        case 17:
+            def = 0;
+            max = 7;
+            break;
+
+        case 19:
+            def = 0;
+            max = 2;
+            break;
+
         case 32:
-            getData->limits.min = -8192;
-            getData->limits.max = 8191;
+            type |= 0x40;
+            min = -8192;
+            def = 0;
+            max = 8191;
             break;
+
+        case 33:
+            type |= 0x40;
+            def = 0;
+            break;
+
+        case 34:
+            def = 0;
+            max = 1;
+            break;
+
+        case 35:
+            type |= 0x40;
+            min = -8;
+            def = 0;
+            max = 7;
+            break;
+
+        case 36:
+            def = 0;
+            max = 3;
+            break;
+
+        case 37:
+            min = -64;
+            def = 0;
+            max = 63;
+            break;
+
+        case 38:
+            type |= 0x40;
+            def = 880;
+            break;
+
+        case 39:
+            type |= 0x40;
+            break;
+
+
         case 48:
         case 49:
+            type |= 0x40;
+            max = 255;
+            break;
+
         case 50:
-            getData->limits.max = 255;
+            type |= 0x40;
+            def = 0;
+            max = 255;
+            break;
+
+        case 51:
+            def = 0;
+            max = 6;
+            break;
+
+        case 64:
+            type |= 0x40;
+            def = 800;
+            break;
+
+        case 65:
+            type |= 0x40;
+            def = 0;
+            break;
+
+        case 66:
+            type |= 0x40;
+            def = 0;
+            break;
+
+        case 67:
+            type |= 0x40;
+            def = 300;
+            break;
+
+        case 68:
+            type |= 0x40;
+            def = 1270;
+            break;
+
+        case 69:
+            def = 0;
+            max = 2;
+            break;
+
+        case 70:
+            def = 0;
+            max = 2;
+            break;
+
+        case 71:
+            type |= 0x40;
+            def = 800;
+            break;
+
+        case 72:
+            type |= 0x40;
+            break;
+
+        case 73:
+        case 74:
+            def = 0;
+            max = 3;
+            break;
+
+        case 75:
+            def = 10;
+            max = 1;
+            break;
+
+        case 80:
+            def = 40;
+            max = 8;
+            break;
+
+        case 81:
+            def = 20;
+            max = 6;
+            break;
+
+        case 82:
+            def = 30;
+            max = 7;
+            break;
+
+        case 83:
+            def = 30;
+            max = 6;
+            break;
+
+        case 104:
+            min = 0;
+            def = 0;
+            max = 0;
+            break;
+
+        case 112:
+            type |= 0x40;
+            def = 10;
+            max = 1;
+            break;
+
+        case 120:
+            type |= 0x40;
+            def = FADEIN_ADJUSTMENT_SCALE * 10;
+            break;
+
+        case 121:
+            type |= 0x40;
+            def = 0;
+            break;
+
+        case 122:
+            type |= 0x40;
+            def = 600;
+            break;
+
+        case 123:
+            type |= 0x40;
+            break;
+
+        case 124:
+            type |= 0x40;
+            def = 720;
+            break;
+
+        default:
+            min = -1;
+            def = -10;
+            max = -1;
             break;
     }
+    getData->data.type |= type;
+    getData->limits.min = min;
+    getData->limits.def = def;
+    getData->limits.max = max;
+}
+
+void PADnoteParameters::postrender(void)
+{
+    // loop over our gathered dirty flags and unset them for the next period
+      AmpLfo->updated
+    = FilterLfo->updated
+    = FreqLfo->updated
+    = false;
+
 }
