@@ -41,7 +41,7 @@ void Presets::setpresettype(const char *type_)
 void Presets::copy(const char *name)
 {
     XMLwrapper *xml = new XMLwrapper(synth);
-
+    bool oldMinimal = xml->minimal;
     // used only for the clipboard
     if (name == NULL)
         xml->minimal = false;
@@ -70,6 +70,7 @@ void Presets::copy(const char *name)
 
     delete(xml);
     nelement = -1;
+    xml->minimal = oldMinimal;
 }
 
 
