@@ -4,7 +4,7 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2009 Nasca Octavian Paul
     Copyright 2009-2011, Alan Calvert
-    Copyright 2014, Will Godfrey
+    Copyright 2014-2017, Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -20,7 +20,8 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    This file is derivative of original ZynAddSubFX code, modified August 2014
+    This file is derivative of original ZynAddSubFX code
+    Modified August 2017
 */
 
 #include <cmath>
@@ -54,10 +55,10 @@ ADnote::ADnote(ADnoteParameters *adpars_, Controller *ctl_, float freq_,
 {
     if (velocity > 1.0f)
         velocity = 1.0f;
-    tmpwavel = (float*)fftwf_malloc(synth->bufferbytes);
-    tmpwaver = (float*)fftwf_malloc(synth->bufferbytes);
-    bypassl = (float*)fftwf_malloc(synth->bufferbytes);
-    bypassr = (float*)fftwf_malloc(synth->bufferbytes);
+    //tmpwavel = (float*)fftwf_malloc(synth->bufferbytes);
+    //tmpwaver = (float*)fftwf_malloc(synth->bufferbytes);
+    //bypassl = (float*)fftwf_malloc(synth->bufferbytes);
+    //bypassr = (float*)fftwf_malloc(synth->bufferbytes);
 
     // Initialise some legato-specific vars
     Legato.msg = LM_Norm;
@@ -83,6 +84,8 @@ ADnote::ADnote(ADnoteParameters *adpars_, Controller *ctl_, float freq_,
         NoteGlobalPar.randpanL = cosf(t * HALFPI);
         NoteGlobalPar.randpanR = cosf((1.0f - t) * HALFPI);
     }
+    else
+        NoteGlobalPar.randpanL = NoteGlobalPar.randpanR = 0.7f;
     NoteGlobalPar.FilterCenterPitch =
         adpars->GlobalPar.GlobalFilter->getfreq() // center freq
         + adpars->GlobalPar.PFilterVelocityScale / 127.0f * 6.0f
@@ -528,6 +531,8 @@ void ADnote::ADlegatonote(float freq_, float velocity_, int portamento_,
         NoteGlobalPar.randpanL = cosf(t * HALFPI);
         NoteGlobalPar.randpanR = cosf((1.0f - t) * HALFPI);
     }
+    else
+        NoteGlobalPar.randpanL = NoteGlobalPar.randpanR = 0.7f;
 
     NoteGlobalPar.FilterCenterPitch =
         adpars->GlobalPar.GlobalFilter->getfreq()
@@ -681,6 +686,8 @@ void ADnote::ADlegatonote(float freq_, float velocity_, int portamento_,
             NoteVoicePar[nvoice].randpanL = cosf(t * HALFPI);
             NoteVoicePar[nvoice].randpanR = cosf((1.0f - t) * HALFPI);
         }
+        else
+            NoteVoicePar[nvoice].randpanL = NoteVoicePar[nvoice].randpanR = 0.7f;
 
         newamplitude[nvoice] = 1.0f;
         if (adpars->VoicePar[nvoice].PAmpEnvelopeEnabled
@@ -840,10 +847,10 @@ ADnote::~ADnote()
 {
     if (NoteEnabled)
         killNote();
-    fftwf_free(tmpwavel);
-    fftwf_free(tmpwaver);
-    fftwf_free(bypassl);
-    fftwf_free(bypassr);
+    //fftwf_free(tmpwavel);
+    //fftwf_free(tmpwaver);
+    //fftwf_free(bypassl);
+    //fftwf_free(bypassr);
     for (int k = 0; k < max_unison; ++k)
         fftwf_free(tmpwave_unison[k]);
     delete [] tmpwave_unison;
@@ -905,6 +912,8 @@ void ADnote::initParameters(void)
             NoteVoicePar[nvoice].randpanL = cosf(t * HALFPI);
             NoteVoicePar[nvoice].randpanR = cosf((1.0f - t) * HALFPI);
         }
+        else
+            NoteVoicePar[nvoice].randpanL = NoteVoicePar[nvoice].randpanR = 0.7f;
 
         newamplitude[nvoice] = 1.0f;
         if (adpars->VoicePar[nvoice].PAmpEnvelopeEnabled)
@@ -1317,6 +1326,48 @@ inline void ADnote::computeVoiceOscillatorLinearInterpolation(int nvoice)
 // end of port
 
 
+/*
+ * Computes the Oscillator (Without Modulation) - CubicInterpolation
+
+ * The differences from the Linear are to little to deserve to be used.
+ * This is because I am using a large zynMaster->getOscilsize(), >512
+ */
+inline void ADnote::computeVoiceOscillatorCubicInterpolation(int nvoice)
+{
+    for (int k = 0; k < unison_size[nvoice]; ++k)
+    {
+        int    poshi = oscposhi[nvoice][k];
+        float  poslo = oscposlo[nvoice][k];
+        int    freqhi = oscfreqhi[nvoice][k];
+        int    freqlo = oscfreqlo[nvoice][k];
+        float  *smps = NoteVoicePar[nvoice].OscilSmp;
+        float  *tw = tmpwave_unison[k];
+        float  xm1, x0, x1, x2, a, b, c;
+        for (int i = 0; i < (synth->p_buffersize); ++i)
+        {
+            xm1 = smps[poshi];
+            x0 = smps[poshi + 1];
+            x1 = smps[poshi + 2];
+            x2 = smps[poshi + 3];
+            a = (3.0 * (x0 - x1) - xm1 + x2) * 0.5;
+            b = 2.0 * x1 + xm1 - (5.0 * x0 + x2) * 0.5;
+            c = (x1 - xm1) * 0.5;
+            tw[i] = (((a * poslo) + b) * poslo + c) * poslo + x0;
+            poslo += freqlo;
+            if (poslo >= 1.0)
+            {
+                poslo -= 1.0;
+                ++poshi;
+            }
+            poshi += freqhi;
+            poshi &= synth->oscilsize - 1;
+        }
+        oscposhi[nvoice][k] = poshi;
+        oscposlo[nvoice][k] = poslo;
+    }
+}
+
+
 // Computes the Oscillator (Morphing)
 void ADnote::computeVoiceOscillatorMorph(int nvoice)
 {
@@ -1632,6 +1683,10 @@ void ADnote::ComputeVoicePinkNoise(int nvoice)
 // Compute the ADnote samples, returns 0 if the note is finished
 int ADnote::noteout(float *outl, float *outr)
 {
+    tmpwavel = synth->getRuntime().genTmp1;
+    tmpwaver = synth->getRuntime().genTmp2;
+    bypassl = synth->getRuntime().genTmp3;
+    bypassr = synth->getRuntime().genTmp4;
     int i, nvoice;
     memset(outl, 0, synth->p_bufferbytes);
     memset(outr, 0, synth->p_bufferbytes);
@@ -1665,7 +1720,12 @@ int ADnote::noteout(float *outl, float *outr)
                         break;
                     default:
                         computeVoiceOscillatorLinearInterpolation(nvoice);
-                        //if (config.cfg.Interpolation) computeVoiceOscillatorCubicInterpolation(nvoice);
+                        /*
+                         * cubic lost something over the years
+                         * it runs but sounds horrible :(
+                         */
+                        //if (synth->getRuntime().Interpolation)
+                            //computeVoiceOscillatorCubicInterpolation(nvoice);
                         break;
                 }
                 break;
