@@ -5,6 +5,7 @@
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011 Alan Calvert
     Copyright 2009 James Morris
+    Copyright 2016 Will Godfrey
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -1611,9 +1612,16 @@ void OscilGen::getLimits(CommandBlock *getData)
 {
     int control = getData->data.control;
     int insert = getData->data.insert;
+
+    // defaults
+    getData->limits.min = 0;
+    getData->limits.max = 127;
+    getData->limits.def = 0;
+
     if (insert > 5)
-    {
-        ; // do harmonics stuff
+    { // do harmonics stuff
+        if (insert == 7)
+            getData->limits.def = 6400;
         return;
     }
     switch (control)
