@@ -4,6 +4,7 @@
     Original ZynAddSubFX author Nasca Octavian Paul
     Copyright (C) 2002-2005 Nasca Octavian Paul
     Copyright 2009-2011 Alan Calvert
+    Copyright 2017 Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -19,7 +20,8 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
-    This file is derivative of ZynAddSubFX original code, modified April 2011
+    This file is derivative of ZynAddSubFX original code
+    Modified March 2017
 */
 #include <cmath>
 
@@ -48,8 +50,6 @@ PADnote::PADnote(PADnoteParameters *parameters, Controller *ctl_, float freq,
     synth(_synth)
 
 {
-//    tmpwave = (float*)fftwf_malloc(synth->bufferbytes);
-
     // Initialise some legato-specific vars
     Legato.msg = LM_Norm;
     Legato.fade.length = (int)truncf(synth->samplerate_f * 0.005f); // 0.005 seems ok.
@@ -127,6 +127,8 @@ PADnote::PADnote(PADnoteParameters *parameters, Controller *ctl_, float freq,
         randpanL = cosf(t * HALFPI);
         randpanR = cosf((1.0f - t) * HALFPI);
     }
+    else
+        randpanL = randpanR = 0.7f;
 
     NoteGlobalPar.FilterCenterPitch =
         pars->GlobalFilter->getfreq() + // center freq
@@ -276,6 +278,8 @@ void PADnote::PADlegatonote(float freq, float velocity,
         randpanL = cosf(t * HALFPI);
         randpanR = cosf((1.0f - t) * HALFPI);
     }
+    else
+        randpanL = randpanR = 0.7f;
 
     NoteGlobalPar.FilterCenterPitch =
         pars->GlobalFilter->getfreq() // center freq
@@ -313,7 +317,6 @@ PADnote::~PADnote()
     delete NoteGlobalPar.GlobalFilterR;
     delete NoteGlobalPar.FilterEnvelope;
     delete NoteGlobalPar.FilterLfo;
-//    fftwf_free(tmpwave);
 }
 
 
