@@ -1,7 +1,7 @@
 /*
     MiscGui.cpp - common link between GUI and synth
 
-    Copyright 2016 Will Godfrey
+    Copyright 2016-2017 Will Godfrey & others
 
     This file is part of yoshimi, which is free software: you can redistribute
     it and/or modify it under the terms of the GNU Library General Public
@@ -17,60 +17,65 @@
     yoshimi; if not, write to the Free Software Foundation, Inc., 51 Franklin
     Street, Fifth Floor, Boston, MA  02110-1301, USA.
 
+    Modified August 2017
 */
 
-#include <FL/Fl.H>
-#include <iostream>
 #include "Misc/SynthEngine.h"
 #include "MiscGui.h"
 #include "MasterUI.h"
 
+#include <iostream>
+
+#include <FL/Fl.H>
+#include <FL/fl_draw.H>
+
+#include <cairo.h>
+#include <cairo-xlib.h>
+
 SynthEngine *synth;
 
 void collect_data(SynthEngine *synth, float value, unsigned char type, unsigned char control, unsigned char part, unsigned char kititem, unsigned char engine, unsigned char insert, unsigned char parameter, unsigned char par2)
 {
-    int typetop = type & 0xc0;
-    if ( part == 0xf1 && insert == 16)
+    int typetop = type & 0xd0;
+    if ( part == 0xf1 && insert == 0x10)
         type |= 8; // this is a hack :(
 
     if (part != 0xd8)
     {
-
-        if ((type & 3) == 3)
-        { // value type is now irrelevant
+        if ((type & 3) == 3 && Fl::event_is_click())
+        {
             if(Fl::event_state(FL_CTRL) != 0)
             {
                 if (type & 8)
-                    type = 3;
+                    type = 3; // previous type is now irrelevant
                 // identifying this for button 3 as MIDI learn
                 else
                 {
-                    synth->getGuiMaster()->midilearnui->words->copy_label("Can't midi-learn this control");
+                    synth->getGuiMaster()->midilearnui->words->copy_label("Can't learn this control");
                     synth->getGuiMaster()->midilearnui->message->show();
+                    synth->getGuiMaster()->midilearnui->message->position(Fl::event_x_root() + 16, Fl::event_y_root());
                     synth->getRuntime().Log("Can't MIDI-learn this control");
                     /* can't use fl_alert here.
                      * For some reason it goes into a loop on spin boxes
                      * and runs menus up to their max value.
-                     *
-                     * fl_alert("Can't MIDI-learn this control");
                      */
                     return;
                 }
             }
             else
-                type = 0;
+                type = 0x40;
                 // identifying this for button 3 as set default
         }
         else if((type & 7) > 2)
-            type = 1 | typetop;
+            type = 1;
             // change scroll wheel to button 1
-
     }
-    type |= (typetop & 0x80);
+    type |= (typetop & 0xd0); // allow for redraws *after* command
+
     CommandBlock putData;
     size_t commandSize = sizeof(putData);
     putData.data.value = value;
-    putData.data.type = type | 0x20; // 0x20 = from GUI
+    putData.data.type = type | 0x20; // = from GUI
     putData.data.control = control;
     putData.data.part = part;
     putData.data.kit = kititem;
@@ -78,7 +83,7 @@ void collect_data(SynthEngine *synth, float value, unsigned char type, unsigned
     putData.data.insert = insert;
     putData.data.parameter = parameter;
     putData.data.par2 = par2;
-
+//cout << "here" << int(type) << " " << int(control) << " " << int(part) << " " << int(parameter) << " " << int(par2) << endl;
     if (jack_ringbuffer_write_space(synth->interchange.fromGUI) >= commandSize)
         jack_ringbuffer_write(synth->interchange.fromGUI, (char*) putData.bytes, commandSize);
 }
@@ -89,7 +94,7 @@ void read_updates(SynthEngine *synth)
     CommandBlock getData;
     size_t commandSize = sizeof(getData);
 
-    while(jack_ringbuffer_read_space(synth->interchange.toGUI) >= commandSize)
+    while (jack_ringbuffer_read_space(synth->interchange.toGUI) >= commandSize)
     {
         int toread = commandSize;
         char *point = (char*) &getData.bytes;
@@ -109,11 +114,17 @@ void decode_updates(SynthEngine *synth, CommandBlock *getData)
     unsigned char insertParam = getData->data.parameter;
     //unsigned char insertPar2 = getData->data.par2;
 
-    if (npart >= 0xc0 && npart < 0xd0) // vector
+    if (npart == 0xe8) // scales
     {
-        return; // todo
+        synth->getGuiMaster()->microtonalui->returns_update(getData);
+        return;
     }
-    if (npart == 0xd8)
+    if (npart == 0xc0) // vector
+    {
+        synth->getGuiMaster()->vectorui->returns_update(getData);
+        return;
+    }
+    if (npart == 0xd8 && synth->getGuiMaster()->midilearnui != NULL)
     {
         synth->getGuiMaster()->midilearnui->returns_update(getData);
         return;
@@ -125,29 +136,68 @@ void decode_updates(SynthEngine *synth, CommandBlock *getData)
     if (kititem >= 0x80 && kititem != 0xff) // effects
     {
         if (npart == 0xf1)
-            synth->getGuiMaster()->syseffectui->returns_update(getData);
+        {
+            if (insert == 1) // dynefilter filter insert
+                synth->getGuiMaster()->syseffectui->fwin_filterui->returns_update(getData);
+            else
+                synth->getGuiMaster()->syseffectui->returns_update(getData);
+        }
         else if (npart == 0xf2)
-            synth->getGuiMaster()->inseffectui->returns_update(getData);
-        else if (npart < 0x40)
-            synth->getGuiMaster()->partui->inseffectui->returns_update(getData);
+        {
+            if (insert == 1) // dynefilter filter insert
+                synth->getGuiMaster()->inseffectui->fwin_filterui->returns_update(getData);
+            else
+                synth->getGuiMaster()->inseffectui->returns_update(getData);
+        }
+        else if (npart < NUM_MIDI_PARTS)
+        {
+            if (insert == 1) // dynefilter filter insert
+                synth->getGuiMaster()->partui->inseffectui->fwin_filterui->returns_update(getData);
+            else
+                synth->getGuiMaster()->partui->inseffectui->returns_update(getData);
+        }
         return;
     }
 
+    if (npart == 0xf8)
+    {
+        synth->getGuiMaster()->configui->returns_update(getData);
+        return;
+    }
     if (npart >= 0xf0) // main / sys / ins
+    {
+        //if (npart == 0xf0) &&  control == 96)
+            //return; // gui in undefined state at this point
+        // doesn't seem to be the case now ???
+        synth->getGuiMaster()->returns_update(getData);
+        return;
+    }
+
+    if (npart >= NUM_MIDI_PARTS)
+        return; // invalid part number
+
+    if (kititem >= NUM_KIT_ITEMS && kititem < 0xff)
+        return; // invalid kit number
+
+    if (kititem == 0xff && engine == 0xff && insert == 0xff && control == 96) // special case for part clear
     {
         synth->getGuiMaster()->returns_update(getData);
         return;
     }
 
-    if (kititem != 0 && engine != 255 && control != 8 && part->kit[kititem & 0x1f].Penabled == false)
+    if (kititem != 0xff && kititem != 0 && engine != 0xff && control != 8 && part->kit[kititem].Penabled == false)
         return; // attempt to access non existant kititem
 
-    if (kititem == 0xff || (kititem & 0x20)) // part
+    if (kititem == 0xff || insert == 0x20) // part
     {
+        if (control != 58 && kititem < 0xff && part->Pkitmode == 0)
+            return; // invalid access
         synth->getGuiMaster()->partui->returns_update(getData);
         return;
     }
 
+    if (kititem > 0 && kititem < 0xff && part->Pkitmode == 0)
+        return; // invalid access
 
     if (engine == 2) // padsynth
     {
@@ -412,29 +462,20 @@ string convert_value(ValueType type, float val)
 
         case VC_LFOfreq:
             f = (powf(2.0f, val * 10.0f) - 1.0f) / 12.0f;
-            if(f<10.0f)
-                return(custom_value_units(f,"Hz", 3));
-            else
-                return(custom_value_units(f,"Hz", 2));
+            return variable_prec_units(f, "Hz", 3);
 
         case VC_LFOdepthFreq: // frequency LFO
             f=powf(2.0f,(int)val/127.0f*11.0f)-1.0f;
-            if (f < 10.0f)
-                return(custom_value_units(f,"cents",2));
-            else if(f < 100.0f)
-                return(custom_value_units(f,"cents",1));
-            else
-                return(custom_value_units(f,"cents"));
+            return variable_prec_units(f, "cents", 2);
+
         case VC_LFOdepthAmp: // amplitude LFO
             return(custom_value_units(val / 127.0f * 200.0f,"%",1));
+
         case VC_LFOdepthFilter: // filter LFO
-            f=(int)val / 127.0f * 4800.0f; // 4 octaves
-            if (f < 10.0f)
-                return(custom_value_units(f,"cents",2));
-            else if(f < 100.0f)
-                return(custom_value_units(f,"cents",1));
-            else
-                return(custom_value_units(f,"cents"));
+            val = (int)val / 127.0f * 4.0f; // 4 octaves
+            f = val * 1200.0f; // cents
+            return variable_prec_units(f, "cents", 2) + "\n("
+                + custom_value_units(val, "base pos. offset)", 2);
 
         case VC_LFOdelay:
             f = ((int)val) / 127.0f * 4.0f + 0.005f;
@@ -450,46 +491,41 @@ string convert_value(ValueType type, float val)
             // unfortunately converttofree() is not called in time for us to
             // be able to use env->getdt(), so we have to compute ourselves
             f = (powf(2.0f, ((int)val) / 127.0f * 12.0f) - 1.0f) * 10.0f;
-            if (f<100.0f)
-                return(custom_value_units(f,"ms",1));
-            else if (f<1000.0f)
-                return(custom_value_units(f,"ms"));
-            else if(f<10000.0f)
-                return(custom_value_units(f/1000.0f,"s",2));
+            if(f >= 1000)
+                return variable_prec_units(f/1000.0f, "s", 2);
             else
-                return(custom_value_units(f/1000.0f,"s",1));
+                return variable_prec_units(f, "ms", 2);
 
         case VC_EnvelopeFreqVal:
             f=(powf(2.0f, 6.0f * fabsf((int)val - 64.0f) / 64.0f) -1.0f) * 100.0f;
             if((int)val<64) f = -f;
-            if(fabsf(f) < 10)
-                return(custom_value_units(f,"cents",2));
-            else if(fabsf(f) < 100)
-                return(custom_value_units(f,"cents",1));
-            else
-                return(custom_value_units(f,"cents"));
+            return variable_prec_units(f, "cents", 2);
 
         case VC_EnvelopeFilterVal:
-            f=((int)val - 64.0f) / 64.0f * 7200.0f; // 6 octaves
-            if(fabsf(f) < 10)
-                return(custom_value_units(f,"cents",2));
-            else if(fabsf(f) < 100)
-                return(custom_value_units(f,"cents",1));
-            else
-                return(custom_value_units(f,"cents"));
+            val = ((int)val - 64.0f) / 64.0f;
+            f = val * 7200.0f; // 6 octaves
+            return variable_prec_units(f, "cents", 2) + "\n("
+                + custom_value_units(val * 6.0f,"base pos. offset)",2);
 
         case VC_EnvelopeAmpSusVal:
             return(custom_value_units((1.0f - (int)val / 127.0f)
                                       * MIN_ENVELOPE_DB, "dB", 1));
 
+        case VC_EnvelopeLinAmpSusVal:
+            f = 20.0f * log10f((int)val / 127.0f);
+            return variable_prec_units(f, "dB", 2);
+
+        case VC_EnvelopeBandwidthVal:
+	    f = powf(2.0f, 10.0f * ((int)val - 64) / 64.0f);
+            return variable_prec_units(f, "x", 4);
+
         case VC_FilterFreq0: // AnalogFilter
             f=powf(2.0f, (val / 64.0f - 1.0f) * 5.0f + 9.96578428f);
-            if (f < 100.0f)
-                return(custom_value_units(f,"Hz",1));
-            else if(f < 1000.0f)
-                return(custom_value_units(f,"Hz"));
+            if (f >= 1000.0f)
+                return variable_prec_units(f/1000.0f, "kHz", 2);
             else
-                return(custom_value_units(f/1000.0f,"kHz",2));
+                return variable_prec_units(f, "Hz", 2);
+
         case VC_FilterFreq2: // SVFilter
             f=powf(2.0f, (val / 64.0f - 1.0f) * 5.0f + 9.96578428f);
             // We have to adjust the freq because of this line
@@ -500,38 +536,30 @@ string convert_value(ValueType type, float val)
             // Using factor 4.0 instead of the usual 2.0*PI leads to a
             // different effective cut-off freq, which we will be showing
             f *= 4.0 / (2.0 * PI);
-            if (f < 100.0f)
-                return(custom_value_units(f,"Hz",1));
-            else if(f < 1000.0f)
-                return(custom_value_units(f,"Hz"));
+            if (f >= 1000.0f)
+                return variable_prec_units(f/1000.0f, "kHz", 2);
             else
-                return(custom_value_units(f/1000.0f,"kHz",2));
+                return variable_prec_units(f, "Hz", 2);
 
-        case VC_FilterFreq1: // ToDo
-            return(custom_value_units(val,""));
+        case VC_FilterFreq1: // Formant filter - base position in vowel sequence
+            return(custom_value_units((val / 64.0f - 1.0f) * 5.0f,"x stretch (modulo 1)",2));
 
         case VC_FilterQ:
         case VC_FilterQAnalogUnused:
             s.clear();
-            s += "Q= ";
+            s += "Q = ";
             f = expf(powf((int)val / 127.0f, 2.0f) * logf(1000.0f)) - 0.9f;
-            if (f<1.0f)
-                s += custom_value_units(f+0.00005f, "", 4);
-            else if (f<10.0f)
-                s += custom_value_units(f+0.005f, "", 2);
-            else if (f<100.0f)
-                s += custom_value_units(f+0.05f, "", 1);
-            else
-                s += custom_value_units(f+0.5f, "");
+            s += variable_prec_units(f, "", 4, true);
             if (type == VC_FilterQAnalogUnused)
                 s += "(This filter does not use Q)";
             return(s);
 
         case VC_FilterVelocityAmp:
-            f = (int)val / 127.0 * -6.0;
-            f = powf(2.0f,f + log(1000.0f)/log(2.0f)); // getrealfreq
+            val = (int)val / 127.0 * -6.0; // formant offset value
+            f = powf(2.0f,val + log(1000.0f)/log(2.0f)); // getrealfreq
             f = log(f/1000.0f)/log(powf(2.0f,1.0f/12.0f))*100.0f; // in cents
-            return(custom_value_units(f-0.5, "cents"));
+            return custom_value_units(f-0.5, "cents") +
+                   "\n(Formant offset: " + custom_value_units(val, "x stretch)",2);
 
         case VC_FilterFreqTrack0:
             s.clear();
@@ -539,6 +567,7 @@ string convert_value(ValueType type, float val)
             f = (val - 64.0f) / 64.0f * 100.0f;
             s += custom_value_units(f, "%", 1);
             return(s);
+
         case VC_FilterFreqTrack1:
             s.clear();
             s += "0/+ checked: range is 0 .. 198%\n";
@@ -546,12 +575,42 @@ string convert_value(ValueType type, float val)
             s += custom_value_units(f, "%", 1);
             return(s);
 
+        case VC_FormFilterClearness:
+            f = powf(10.0f, (val - 32.0f) / 48.0f);
+            return custom_value_units(f, " switch rate",2);
+
+        case VC_FormFilterSlowness:
+            f = powf(1.0f - (val / 128.0f), 3.0f);
+            return custom_value_units(f, " morph rate",4);
+
+        case VC_FormFilterStretch:
+            f = powf(0.1f, (val - 32.0f) / 48.0f);
+            return custom_value_units(f, " seq. scale factor",3);
+
         case VC_InstrumentVolume:
             return(custom_value_units(-60.0f*(1.0f-(int)val/96.0f),"dB",1));
 
         case VC_ADDVoiceVolume:
             return(custom_value_units(-60.0f*(1.0f-lrint(val)/127.0f),"dB",1));
 
+        case VC_ADDVoiceDelay:
+            if((int) val == 0)
+                return "No delay";
+            f = (expf((val/127.0f) * logf(50.0f)) - 1) / 10;
+            if(f >= 1)
+                return variable_prec_units(f, "s", 2, true);
+            else
+                return variable_prec_units(f * 1000, "ms", 1);
+
+        case VC_PitchBend:
+            if ((int) val == 64)
+                return "Off - no pitch bend";
+            f = (val - 64) / 24;
+            s = string(f > 0 ? "" : "\n(reversed)");
+            f = fabsf(f);
+                return custom_value_units(f, "x bend range " + s, 2) +
+                    "\n(default: +/- " + custom_value_units(200 * f, "cents )");
+
         case VC_PartVolume:
             return(custom_value_units((val-96.0f)/96.0f*40.0f,"dB",1));
 
@@ -565,6 +624,7 @@ string convert_value(ValueType type, float val)
                 return(custom_value_units((64.0f - i) / 63.0f * 100.0f,"% left"));
             else
                 return(custom_value_units((i - 64.0f)/63.0f*100.0f,"% right"));
+
         case VC_PanningStd:
             i = lrint(val);
             if(i==64)
@@ -604,6 +664,15 @@ string convert_value(ValueType type, float val)
             f = 15.0f*(f * sqrtf(fabsf(f)));
             return(custom_value_units(f+((f<0) ? (-0.005f) : (0.005f)),"Hz",2));
 
+        case VC_FixedFreqET:
+            f = powf(2.0f, (lrint(val) - 1) / 63.0f) - 1.0f;
+            if(lrint(val) <= 1) /* 0 and 1 are both fixed */
+                return "Fixed";
+            else if(lrint(val) <= 64)
+                return custom_value_units(powf(2.0f,f),"x /octave up",2);
+            else
+                return custom_value_units(powf(3.0f,f),"x /octave up",2);
+
         case VC_FilterGain:
             f = ((int)val / 64.0f -1.0f) * 30.0f; // -30..30dB
             f += (f<0) ? -0.05 : 0.05;
@@ -621,21 +690,35 @@ string convert_value(ValueType type, float val)
             // Max dB range for vel=1 compared to vel=127
             s += "Velocity Dynamic Range ";
             f = -20.0f * logf(powf((1.0f / 127.0f), f)) / log(10.0f);
-            if (f < 100.0f)
-                s += custom_value_units(f,"dB",1);
-            else
-                s += custom_value_units(f,"dB");
+            s += variable_prec_units(f, "dB", 2);
             s += "\nVelocity/2 = ";
-            s += custom_value_units(f/-6.989f,"dB",1); // 6.989 is log2(127)
+            s += variable_prec_units(f/(-1 * log2(127)), "dB", 2);
             return(s);
 
         case VC_BandWidth:
             f = powf((int)val / 1000.0f, 1.1f);
             f = powf(10.0f, f * 4.0f) * 0.25f;
-            if (f<10.0f)
-                return(custom_value_units(f,"cents",2));
-            else
-                return(custom_value_units(f,"cents",1));
+            return variable_prec_units(f, "cents", 2);
+
+        case VC_SubBandwidth:
+            /* This is only an approximation based on observation.
+               Considering the variability of the synthesis depending
+               on number of filter stages, it seems accurate enough.
+             */
+	    f = powf(10.0f, (val - 127.0f) / 127.0f * 4.0f) * 4800;
+            return variable_prec_units(f, "cents", 3);
+
+        case VC_SubBandwidthRel:
+	    f = powf(100.0f, (63 - (int)val) / 64.0f);
+            return variable_prec_units(f, "x", 3);
+
+        case VC_SubBandwidthScale:
+            if((int)val == 0)
+                return "Constant";
+	    f = val / 64.0f * 3.0f;
+            return "Factor (100,10k): " +
+                variable_prec_units(powf(10,f), "", 4) + ", " +
+                variable_prec_units(powf(0.1,f), "x", 4);
 
         case VC_FilterVelocitySense: // this is also shown graphically
             if((int)val==127)
@@ -712,10 +795,7 @@ string convert_value(ValueType type, float val)
 
         case VC_FXReverbTime:
             f = powf(60.0f, (int)val / 127.0f) - 0.97f; // s
-            if (f<10.0f)
-                return(custom_value_units(f+0.005,"s",2));
-            else
-                return(custom_value_units(f+0.05,"s",1));
+            return variable_prec_units(f, "s", 2, true);
 
         case VC_FXReverbIDelay:
             f = powf(50.0f * (int)val / 127.0f, 2.0f) - 1.0f; // ms
@@ -769,12 +849,7 @@ string convert_value(ValueType type, float val)
 
         case VC_FXReverbBandwidth:
             f = powf((int)val / 127.0f, 2.0f) * 200.0f; // cents
-            if(f<1.0f)
-                return(custom_value_units(f+0.005,"cents",2));
-            else if(f<100.0f)
-                return(custom_value_units(f+0.05,"cents",1));
-            else
-                return(custom_value_units(f+0.5,"cents"));
+            return variable_prec_units(f, "cents", 2, true);
 
         case VC_FXdefaultVol:
             f = ((int)val / 127.0f)*1.414f;
@@ -783,24 +858,15 @@ string convert_value(ValueType type, float val)
 
         case VC_FXlfofreq:
             f = (powf(2.0f, (int)val / 127.0f * 10.0f) - 1.0f) * 0.03f;
-            if(f<10.0f)
-                return(custom_value_units(f,"Hz", 3));
-            else
-                return(custom_value_units(f,"Hz", 2));
+            return variable_prec_units(f, "Hz", 3);
 
         case VC_FXChorusDepth:
             f = powf(8.0f, ((int)val / 127.0f) * 2.0f) -1.0f; //ms
-            if(f<10.0f)
-                return(custom_value_units(f+0.005,"ms",2));
-            else
-                return(custom_value_units(f+0.05,"ms",1));
+            return variable_prec_units(f, "ms", 2, true);
 
         case VC_FXChorusDelay:
             f = powf(10.0f, ((int)val / 127.0f) * 2.0f) -1.0f; //ms
-            if(f<1.0f)
-                return(custom_value_units(f+0.005,"ms",2));
-            else
-                return(custom_value_units(f+0.05,"ms",1));
+            return variable_prec_units(f, "ms", 2, true);
 
         case VC_FXdefaultFb:
             f = (((int)val - 64.0f) / 64.1f) * 100.0f;
@@ -836,22 +902,14 @@ string convert_value(ValueType type, float val)
 
         case VC_FXEQfreq:
             f = 600.0f * powf(30.0f, ((int)val - 64.0f) / 64.0f);
-            if (f < 100.0f)
-                return(custom_value_units(f,"Hz",1));
-            else if(f < 1000.0f)
-                return(custom_value_units(f,"Hz"));
+            if (f >= 1000)
+                return variable_prec_units(f/1000.f, "kHz", 2);
             else
-                return(custom_value_units(f/1000.0f,"kHz",2));
+                return variable_prec_units(f, "Hz", 2, true);
 
         case VC_FXEQq:
             f = powf(30.0f, ((int)val - 64.0f) / 64.0f);
-            if (f<1.0f)
-                s += custom_value_units(f+0.00005f, "", 4);
-            else if (f<10.0f)
-                s += custom_value_units(f+0.005f, "", 2);
-            else
-                s += custom_value_units(f+0.05f, "", 1);
-            return(s);
+            return variable_prec_units(f, "", 3, true);
 
         case VC_FXEQgain:
             f = 20.0f - 46.02f*(1.0f - ((int)val / 127.0f));
@@ -866,8 +924,14 @@ string convert_value(ValueType type, float val)
             return(custom_value_units(f,"dB",1));
 
         case VC_plainValue:
-            return(custom_value_units(val,""));
-
+        {
+            /* Avoid trailing space */
+            ostringstream oss;
+            oss.setf(std::ios_base::fixed);
+            oss.precision(0);
+            oss << val;
+            return string(oss.str());
+        }
         case VC_FXDistVol:
             f = -40.0f * (1.0f - ((int)val / 127.0f)) + 15.05f;
             return(custom_value_units(f,"dB",1));
@@ -894,62 +958,221 @@ string convert_value(ValueType type, float val)
     return(custom_value_units(val,""));
 }
 
-int custom_graph_size(ValueType vt)
+void custom_graph_dimensions(ValueType vt, int& w, int& h)
 {
     switch(vt)
     {
-        case VC_FilterVelocitySense:
-            return(48);
-        default:
-            return(0);
+    case VC_FilterVelocitySense:
+        w = 128;
+        h = 64;
+        break;
+    case VC_SubBandwidthScale:
+        w = 256;
+        h = 128;
+        break;
+    case VC_FormFilterClearness:
+        w = 128;
+        h = 128;
+        break;
+    default:
+        w = 0;
+        h = 0;
     }
 }
 
+inline void grid(int x, int y, int w, int h, int sections)
+{
+        fl_color(FL_GRAY);
+
+        int j = 1;
+        int gDist = h / sections;
+        for(; j < sections; j++) /* Vertical */
+        {
+            fl_line(x, y - gDist * j, x + w, y - gDist * j);
+        }
+
+        gDist = w / sections;
+        for(j = 1; j < sections; j++) /* Horizontal */
+        {
+            fl_line(x + gDist * j, y, x + gDist * j, y - h);
+        }
+}
+
 void custom_graphics(ValueType vt, float val,int W,int H)
 {
-    int size,x0,y0,i;
+    int x0,y0,i;
+    int _w, _h;
     float x,y,p;
+    custom_graph_dimensions(vt, _w, _h);
+    x0 = W / 2 - (_w / 2);
+    y0 = H;
 
     switch(vt)
     {
-        case VC_FilterVelocitySense:
-            size=42;
-            x0 = W-(size+2);
-            y0 = H-(size+2);
-
-            fl_color(215);
-            fl_rectf(x0,y0,size,size);
-            fl_color(FL_BLUE);
-
-            p = powf(8.0f,(64.0f-(int)val)/64.0f);
-            y0 = H-3;
+    case VC_FilterVelocitySense:
+    {
+        p = powf(8.0f,(64.0f-(int)val)/64.0f);
 
+        /* Grid */
+        grid(x0,y0,_w,_h, 4);
+        /*Function curve*/
+        fl_color(FL_BLUE);
+        if ((int)val == 127)
+        {   // in this case velF will always return 1.0
+            y = y0 - _h;
+            fl_line(x0, y, x0 + _w, y);
+        }
+        else
+        {
             fl_begin_line();
-
-            size--;
-            if ((int)val == 127)
-            {   // in this case velF will always return 1.0
-                y = 1.0f * size;
-                for(i=0;i<=size;i++)
-                {
-                    x = (float)i / (float)size;
-                    fl_vertex((float)x0+i,(float)y0-y);
-                }
-            }
-            else
+            for(i = 0; i < _w; i++)
             {
-                for(i=0;i<=size;i++) {
-                    x = (float)i / (float)size;
-                    y = powf(x,p) * size;
-                    fl_vertex((float)x0+i,(float)y0-y);
-                }
+                x = (float)i / (float)_w;
+                y = powf(x,p) * _h;
+                fl_vertex((float)x0 + i, (float)y0 - y);
             }
             fl_end_line();
-            break;
-
-        default:
-            break;
+        }
+        break;
     }
+    case VC_FormFilterClearness:
+    {
+        p = powf(10.0f, (val - 32.0f) / 48.0f); //clearness param
+        grid(x0,y0,_w,_h,10);
+        fl_color(FL_BLUE);
+        fl_begin_line();
+        x = 0;
+        float frac = 1.0f / (float)_w;
+        for(i = 0; i < _w; i++)
+        {
+            y = (atanf((x * 2.0f - 1.0f) * p) / atanf(p) + 1.0f) * 0.5f * _h;
+            fl_vertex((float)x0 + i, (float)y0 - y);
+            x += frac;
+        }
+        fl_end_line();
+        break;
+    }
+    case VC_SubBandwidthScale:
+    {
+        /* The scale centers around the factor 1 vertically
+           and is logarithmic in both dimensions. */
+
+        int margin = 28;
+        _h -= margin;
+        _w -= margin * 2;
+        x0 += margin * 1.25;
+        y0 -= margin * 0.75;
+
+        float cy = y0 - _h / 2;
+
+        int j = 1;
+        const float lg1020 = log10(20); /* Lower bound = 20hz*/
+        const float rx = _w / (log10(20000) - lg1020); /* log. width ratio */
+        const float ry = (_h / 2) / log10(100000);
+
+        string hzMarkers[] = {"20", "100", "1k", "10k"};
+        string xMarkers[] = {"x10","x100","x1k","x10k","10%","1%","0.1%","0.01%"};
+
+        /* Scale lines */
+
+        fl_font(fl_font(),8);
+        fl_line_style(0);
+        for(i = 0; i < 4; i++) /* 10x / 10%, 100x / 1% ... */
+        {
+            y = ry * (i + 1);
+            fl_color(169,169,169);
+            fl_line(x0, cy - y, x0 + _w, cy - y);
+            fl_line(x0, cy + y, x0 + _w, cy + y);
+            fl_color(0,0,0);
+            fl_draw(xMarkers[i].c_str(), x0 - 28, (cy - y - 4), 24, 12,
+                    Fl_Align(FL_ALIGN_RIGHT));
+            fl_draw(xMarkers[i + 4].c_str(), x0 - 28, (cy + y - 4), 24, 12,
+                    Fl_Align(FL_ALIGN_RIGHT));
+        }
+
+        /* Hz lines */
+
+        fl_color(196,196,196); /* Lighter inner lines*/
+
+        for(i = 10;i != 0; i *= 10)
+        {
+            for(j = 2; j < 10; j++)
+            {
+                x = x0 + rx * (log10(i * j) - lg1020) + 1;
+                fl_line(x, y0, x, y0 - _h);
+                if(i * j >= 20000)
+                {
+                    i = 0;
+                    break;
+                }
+            }
+        }
+
+        fl_font(fl_font(),10);
+        for(i = 0; i < 4; i++) /* 20, 100, 1k, 10k */
+        {
+            x = x0 + (i == 0 ?  0 : ((float)i + 1 - lg1020) * rx);
+            fl_color(127,127,127); /* Darker boundary lines */
+            fl_line(x, y0, x, y0 - _h);
+            fl_color(FL_BLACK);
+            fl_draw(hzMarkers[i].c_str(), x - 20, y0 + 4, 40, 12,
+                    Fl_Align(FL_ALIGN_CENTER));
+        }
+        /* Unit marker at the lower right of the graph */
+        fl_draw("Hz", x0 + _w, y0 + 4, 20, 12, Fl_Align(FL_ALIGN_LEFT));
+
+        /* Vertical center line */
+        fl_color(64,64,64);
+        fl_line(x0 - margin, cy, x0 + _w, cy);
+
+        /* Function curve */
+        fl_color(FL_BLUE);
+        if((int)val == 0)
+        {
+            fl_line(x0, cy, x0 + _w, cy);
+        }
+        else
+        {
+            const float p = ((int)val / 64.0f) * 3.0;
+
+            /* Cairo not necessary, but makes it easier to read the graph */
+            cairo_t *cr;
+            cairo_surface_t* Xsurface = cairo_xlib_surface_create
+                (fl_display, fl_window, fl_visual->visual,
+                 Fl_Window::current()->w(), Fl_Window::current()->h());
+            cr = cairo_create (Xsurface);
+
+            cairo_set_source_rgb(cr, 1, 0, 0);
+            cairo_set_line_width(cr, 1.5);
+            cairo_move_to(cr, x0, cy - ry * log10(powf(50, p)));
+            cairo_line_to(cr, x0 + _w, cy - ry * log10(powf(0.05, p)));
+            cairo_stroke(cr);
+
+            cairo_surface_destroy(Xsurface);  cairo_destroy(cr);
+        }
+        break;
+    }
+    default:
+        break;
+    }
+}
+
+string variable_prec_units(float v, string u, int maxPrec, bool roundup)
+{
+    int digits = 0, lim = (int) pow(10,maxPrec);
+    float _v = fabsf(v);
+    while(maxPrec > digits)
+    {
+        if(_v >= lim)
+            break;
+        digits++;
+        lim /= 10;
+    }
+    if (roundup)
+    {
+        v += 5 * powf(10,-(digits + 1));
+    }
+    return custom_value_units(v, u, digits);
 }
 
 string custom_value_units(float v, string u, int prec)
